/********************************************************************/

#ifndef __GENINITdotH__
#define __GENINITdotH__

/********************************************************************/

// These routines the Gen library to set services up before an
// application starts, and after it has executed. 
//
// The init method should be the first code executed in main, before
// any other code. 
//
// The shutdown method should be the last code executed in main, so that
// it can clean up anything that the init method did.
CFUNC void genInit (int argc , char **argv);
CFUNC void genShutdown (void);

// Determine the project root (directory under the directory that the current
// binary is being run in) and return it as a pointer to a static buffer.
CFUNC const CHAR *getProjectRootCSTRING (void);

// Determine the project version (specified in Makefiles/ReleaseNumber) and
// return it as a pointer to a static buffer.
CFUNC const CHAR *getProjectVersionCSTRING (void);

// If the file including this header isn't being compiled as ObjC code, then
// define these variables, which allow C and C++ to refer to these functions
// with more natural names. The net effect is that the API is the same for
// all languages, with ObjC having an additional CSTRING version of the function.
#ifndef __OBJC__

// Use defines to make C and C++ give the CSTRING version of these two functions
// a more natural name.
#define getProjectRoot() getProjectRootCSTRING ()
#define getProjectVersion() getProjectVersionCSTRING ()

#endif

/********************************************************************/

// If we are compiling this code as ObjC, then include in some definitions for
// ObjC functions that we might want to call.
#ifdef __OBJC__

// Determine the project root (directory under the directory that the current
// binary is being run in) and return it as an NSString. 
OBJCFUNC NSString *getProjectRoot (void);

// Determine the project version (specified in Makefiles/ReleaseNumber) and
// return it as an NSString.
OBJCFUNC NSString *getProjectVersion (void);

#endif

/********************************************************************/

// If we are compiling an ObjC file, or are compiling C/C++ and are 
// configured to include ObjC support, then include prototypes for
// init functions that the C versions will call to setup ObjC specific
// stuff for us.
//
// If we are compiling C/C++ and NOT configured to support ObjC, then
// these functions will become macros that expand out to nothing so that
// the same code compiles both ways and we let the #ifdef hackery be
// here in the header instead of the source (which is cleaner).
#if defined (__USING_OBJC__) || defined (__OBJC__)

// We are compiling an ObjC file, or are compiling C/C++ and configured to
// support ObjC. So, include prototypes for these functions.
OBJCFUNC void genInitObjC (int argc , char **argv);
OBJCFUNC void genShutdownObjC (void);

#else

// Since we're not supporting ObjC, define these functions to be macros which
// will expand to nothing.
#define genInitObjC(argc,argv)
#define genShutdownObjC()

#endif

/********************************************************************/

#endif

