/********************************************************************/

#include "Gen.h"

/********************************************************************/

const char *_FILE_;
const char *_FUNCTION_;
INT4 _LINE_;

/********************************************************************/

// Actually log a message.
static void logAMessage (FILE *out , const char *type , const char *fmt , va_list args)
{
    static const char *lastFile;
    static const char *lastFunction;
    static INT4 lastLine;
    int printed;

    printed = fprintf (out , "%s: " , type);
    printed += vfprintf (out , fmt , args);

    if (lastFile != _FILE_ || lastFunction != _FUNCTION_ || lastLine != _LINE_)
    {
        // If we printed out a small amount of information, then spit out
        // some seperating space between the message and the location,
        // so things don't look so crowded.
        if (printed < 80)
        {
            while (printed++ < 80)
                fprintf (out , " ");
        }

        // Something changed, so output it
        fprintf (out , " (");

        // If the file is different, output it.
        if (lastFile != _FILE_)
            fprintf (out , "%s" , _FILE_);
        else
            fprintf (out , "^");

        // if the line is different, output it
        if (lastLine != _LINE_)
            fprintf (out , ":%d" , _LINE_);

        // if the function is different, output it
        if (lastFunction != _FUNCTION_)
            fprintf (out , " in %s" , _FUNCTION_);

        fprintf (out , ")\n");
        
        // Save our state now
        lastFile = _FILE_;
        lastFunction = _FUNCTION_;
        lastLine = _LINE_;
    } else {
        fprintf (out , "\n");
    }
}

/********************************************************************/

// Logging functions
void MessageLog_ (const char *fmt , ...)
{
    va_list args;

    va_start (args , fmt);
    logAMessage (stdout , "Msg " , fmt , args);
    va_end (args);
}

/********************************************************************/

void WarningLog_ (const char *fmt , ...)
{
    va_list args;

    va_start (args , fmt);
    logAMessage (stdout , "Warn" , fmt , args);
    va_end (args);
}

/********************************************************************/

void ErrorLog_ (const char *fmt , ...)
{
    va_list args;

    va_start (args , fmt);
    logAMessage (stderr , "Err " , fmt , args);
    va_end (args);
}

/********************************************************************/

void FatalErrorLog_ (const char *fmt , ...)
{
    va_list args;

    va_start (args , fmt);
    logAMessage (stderr , "FErr" , fmt , args);
    va_end (args);
}

/********************************************************************/

void ProgrammerErrorLog_ (const char *fmt , ...)
{
    va_list args;

    va_start (args , fmt);
    logAMessage (stderr , "PErr" , fmt , args);
    va_end (args);
}

/********************************************************************/


