/********************************************************************/

#ifndef __GENSETUPdotH__
#define __GENSETUPdotH__

/********************************************************************/

// Standard include files
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>
#include <sys/vfs.h>
#include <sys/time.h>
#include <sys/stat.h>

// If we're compiling ObjC, pull in the foundation libraries as well.
#ifdef __OBJC__
#import <Foundation/Foundation.h>
#endif

/********************************************************************/

// The values for BOOL. libobjc has it's own ideas about YES and NO
// that we want to override, since there is more values that should be
// YES than just one.
#ifdef NO
#undef NO
#endif

#ifdef YES
#undef YES
#endif

#define NO 0
#define YES (!NO)

#define SUCCESS	YES
#define FAILURE	NO

#define TRUE YES
#define FALSE NO

// This macro saves the current compile location for use in logging.
#define STORE_LINE _FILE_ = __FILE__, _LINE_=__LINE__, _FUNCTION_=__FUNCTION__

// The functions for logging. These are slightly different for ObjC than they
// are for C code, so that we can have two versions of each and have the same
// library work for both languages at the same time.
#ifdef __OBJC__

#define MessageLog         STORE_LINE, MessageLog_ObjC_
#define WarningLog         STORE_LINE, WarningLog_ObjC_
#define ErrorLog           STORE_LINE, ErrorLog_ObjC_
#define FatalErrorLog      STORE_LINE, FatalErrorLog_ObjC_
#define ProgrammerErrorLog STORE_LINE, ProgrammerErrorLog_ObjC_

#else

#define MessageLog         STORE_LINE, MessageLog_ 
#define WarningLog         STORE_LINE, WarningLog_
#define ErrorLog           STORE_LINE, ErrorLog_
#define FatalErrorLog      STORE_LINE, FatalErrorLog_
#define ProgrammerErrorLog STORE_LINE, ProgrammerErrorLog_

#endif

// Based on what language this header is being included as, set up the defines
// for each language. This is really only neccesary to make sure that C++ things
// that C functions are C functions (so that it will link properly). But we can
// use this to mark functions in the library header files with the language they're
// based in, which makes reading the stuff cleaner.
#if defined (__C__) || defined (__OBJC__)

// These languages share the standard C linkage.
#define CFUNC
#define CPPFUNC 
#define OBJCFUNC

#else

// As far as C++ is concerned, C and ObjC functions have the C linkage, so set
// that here, so that C++ can call our functions.
#define CFUNC    extern "C"
#define CPPFUNC
#define OBJCFUNC extern "C"

#endif

/********************************************************************/

// Standard types
typedef char *          PTR;
typedef char *          CSTRING;
typedef char	        CHAR;
typedef unsigned char	UCHAR;
typedef unsigned char   BYTE;
typedef BYTE *          BINARY;

// We only need to make this typedef if we're not compiling __OBJC__,
// since libobjc makes it for us.
#ifndef __OBJC__

// Just in case there is already a define, undefine it.
#ifdef BOOL
#undef BOOL
#endif

typedef unsigned char   BOOL;


#endif

// Types of guaranteed size
typedef char               INT1;
typedef unsigned char      UINT1;
typedef short              INT2;
typedef unsigned short     UINT2;
typedef int                INT4;
typedef unsigned int       UINT4;
typedef long long          INT8;
typedef unsigned long long UINT8;

// Aliases for floating point types
typedef float            FLOAT;
typedef double           DOUBLE;
typedef DOUBLE           DATE;

// The type for dates that are stored as epoch
// time instead of NSTimeInterval
typedef INT4             CDATE;

/********************************************************************/

// Stores the current file, line and function for a log. Doesn't
// work so good in multithreaded code, whatcha gonna do?
extern const char *_FILE_, *_FUNCTION_;
extern INT4 _LINE_;

/********************************************************************/

// If we're ObjC, then we want the versions of these functions that
// can take an NSString as the first argument.
#ifdef __OBJC__

// Logging functions
void MessageLog_ObjC_ (NSString *fmt , ...);
void WarningLog_ObjC_ (NSString *fmt , ...);
void ErrorLog_ObjC_ (NSString *fmt , ...);
void FatalErrorLog_ObjC_ (NSString *fmt , ...);
void ProgrammerErrorLog_ObjC_ (NSString *fmt , ...);

#else 

// In here, we're C or C++. If we're compiling C++ code, make sure it 
// knows that the following functions are C functions, so C++ can link 
// with us.
#ifdef __CPP__
extern "C" {
#endif

// Logging functions
void MessageLog_ (const char *fmt , ...);
void WarningLog_ (const char *fmt , ...);
void ErrorLog_ (const char *fmt , ...);
void FatalErrorLog_ (const char *fmt , ...);
void ProgrammerErrorLog_ (const char *fmt , ...);

// Done the extern C now.
#ifdef __CPP__
}
#endif

// #ifdef __OBJC__
#endif

/********************************************************************/

#endif

