/*****************************************************************************/

#include <Gen/Gen.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
#include "environment.h"

/*****************************************************************************/

/* This defines all of the environment variables that will be pulled and
 * placed into the CGI table upon startup. The first is the actual name of
 * the environment variable, and the second is it's internal name in the
 * table. Variables that don't exist at the time the script executes will be
 * silently dropped. Terminate the list with NULL. 
 *
 * Others will perhaps want to modify this so that the key name and the
 * environment name are the same. I consider this a matter of personal
 * preference, and even though I think constant defines should have
 * CAPITAL_NAMES, it just seems wrong to do that in this context. */
const char *vars[] = {
    "USER"                 , "user"               ,
    "HOME"                 , "userHome"           ,
    "AUTH_TYPE"            , "authType"           ,
    "CONTENT_LENGTH"       , "contentLength"      ,
    "CONTENT_TYPE"         , "contentType"        ,
    "DOCUMENT_ROOT"        , "documentRoot"       ,
    "GATEWAY_INTERFACE"    , "gatewayInterface"   ,
    "HTTP_ACCEPT"          , "httpAccept"         ,
    "HTTP_ACCEPT_ENCODING" , "httpAcceptEncoding" ,
    "HTTP_ACCEPT_LANGUAGE" , "httpAcceptLanguage" ,
    "HTTP_HOST"            , "httpHost"           ,
    "HTTP_USER_AGENT"      , "httpUserAgent"      ,
    "HTTP_COOKIE"          , "httpCookie"         ,
    "PATH_INFO"            , "pathInfo"           ,
    "PATH_TRANSLATED"      , "pathTranslated"     ,
    "QUERY_STRING"         , "queryString"        ,
    "REMOTE_ADDR"          , "remoteAddr"         ,
    "REMOTE_HOST"          , "remoteHost"         ,
    "REMOTE_IDENT"         , "remoteIdent"        ,
    "REMOTE_USER"          , "remoteUser"         ,
    "REQUEST_METHOD"       , "requestMethod"      ,
    "REQUEST_URI"          , "requestURI"         ,
    "SCRIPT_FILENAME"      , "scriptFilename"     ,
    "SCRIPT_NAME"          , "scriptName"         ,
    "SERVER_ADDR"          , "serverAddr"         ,
    "SERVER_ADMIN"         , "serverAdmin"        ,
    "SERVER_NAME"          , "serverName"         ,
    "SERVER_PORT"          , "serverPort"         ,
    "SERVER_PROTOCOL"      , "serverProtocol"     ,
    "SERVER_SIGNATURE"     , "serverSignature"    ,
    "SERVER_SOFTWARE"      , "serverSoftware"     ,
    NULL
}; 

/*****************************************************************************/

/* Open up the environment library. */
LUALIB_API void lua_environmentlibopen (lua_State *L)
{
    int varNum;

    /* First thing to do is put a new empty table onto the stack. */
    lua_newtable (L);

    /* Now, we will scan through the list of environment variables, and
     * set any we find. We make sure not to set an empty variable. Hope
     * that's not bad? */
    for (varNum = 0 ; vars[varNum] != NULL ; varNum += 2)
    {
        /* Get the value of the variable */
        char *value = getenv (vars[varNum]);

        /* If we got a value, and it's not empty, put it into the
         * table. */
        if (value != NULL && value[0] != '\0')
        {
            /* Push first the key, then the value, then set it into the
             * table. The call to set the table pops the key and value 
             * from the stack. */
            lua_pushstring (L , vars[varNum + 1]);
            lua_pushstring (L , value);
            lua_settable (L , -3);
        }
    }

    /* Now that we're all done, we need to set a global variable using
     * this table. This is, fortunately, easy. */
    lua_setglobal (L , "env");
}

/*****************************************************************************/

