/*****************************************************************************/

#define _XOPEN_SOURCE
#include <Gen/Gen.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

/*****************************************************************************/

/* This contains the set [a-zA-Z0-9./] which has been randomized. This makes
 * 64 characters. */
static char *randomChars = "Al1OUqmY4seJ87ncZQfazd/TbNE3SwjrGhKVtRok5IpF9.g2x6PuBDMiCXvyW0HL";

/*****************************************************************************/

/* crypt (key , [salt])
 *
 * Crypt the given key using the internal crypt function. If given, salt is
 * a value used to perturb the DES crypt algorithm. If it is not specified, 
 * a random salt is obtained. If the salt is less than 2 characters long,
 * it will be expanded out to be 2 characters long. If the salt is longer than
 * 2 characters, only the first two characters will be used.
 *
 * The return value is a string which represents the encrypted key. The first
 * two characters of the returned string are the salt which was used to crypt
 * it in the first place. On error, nil is returned. This means that the libc 
 * that was linked with does not contain crypt functionality.
 *
 * There is no decrypt functionality. Instead, your only option is to crypt the
 * same input a second time using the same salt, and compare to see if you get
 * the same value out as you got the first time around. */
int ext_crypt (lua_State *L)
{
    /* Get the key and salt. The salt is optional, and will come across as an
     * empty string if it wasn't given. */
    const char *key = luaL_check_lstr (L , 1 , NULL);
    const char *salt = luaL_opt_lstr (L , 2 , "" , NULL);
    const char *result;
    char realSalt[3] = "";

    /* Copy the salt over for manipulation. */
    strcpy (realSalt , salt);

    /* Add random characters to the salt until it's length is 2. */
    while (strlen (realSalt) < 2)
    {
        /* Get a random character from the randomChars string, and
         * append it to the salt. */
        char temp[2] = " ";
        temp[0] = randomChars[rand () % 64];
        strcat (realSalt , temp);
    }

    /* If the salt is too long, cap it. */
    if (strlen (realSalt) > 2)
        realSalt[2] = '\0';

    /* Do the crypt. */
    result = crypt (key , realSalt);

    /* If it worked, push the string. Otherwise, push nil. */
    if (result == NULL)
        lua_pushnil (L);
    else
        lua_pushstring (L , result);
    return 1;
}

/*****************************************************************************/

