/*****************************************************************************/

#include <Gen/Gen.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
#include <graphics/graphics.h>
#include <environment/environment.h>
#include <query/query.h>
#include <cookie/cookie.h>
#include <extensions/extensions.h>
#include <luadb/luadb.h>
#include <html/html.h>

/*****************************************************************************/

/* Entry point of the program. Duh. */
int main (int argc , char **argv)
{
    lua_State *L;

    /* Check to see if we were called properly or not. */
    if (argc < 2)
    {
        printf ("Content-Type: text/html\n\n");
        printf ("You're supposed to pass me a script when you call me.\n");
        return 1;
    }
    
    /* We must be good to go. Firstly, open up a lua state. */
    L = lua_open ();

    /* Initialize all of the standard libraries. Each of these functions may
     * leave something on the stack. So to get around that, after we open
     * each library, we must clean the stack. */
    luaopen_base (L);    lua_settop (L , 0);
    luaopen_table (L);   lua_settop (L , 0);
    luaopen_io (L);      lua_settop (L , 0);
    luaopen_string (L);  lua_settop (L , 0);
    luaopen_math (L);    lua_settop (L , 0);
    luaopen_debug (L);   lua_settop (L , 0);
    luaopen_loadlib (L); lua_settop (L , 0);

    /* Now initialize our libraries. */
    lua_extensionslibopen (L);
    lua_graphicslibopen (L);
    lua_environmentlibopen (L);
    lua_querylibopen (L);
    lua_cookielibopen (L);
    lua_luadblibopen (L);
    lua_htmllibopen (L);

    /* Set up our version. */
    lua_pushstring (L , getProjectVersion ());
    lua_setglobal (L , "luaweb_version");

    /* Now we can execute the script we were passed. */
    return lua_dofile (L , argv[1]);
}

/*****************************************************************************/
