#ifndef __QUERYdotH__
#define __QUERYdotH__

/*****************************************************************************/

/* These define how big a key and a value in a Query String can be. Keys and 
 * values larger than these sizes get truncated during parsing. Generally
 * speaking, low values are good for keys because they're not likely to be
 * very long, while value should be fairly big because form fields like text
 * area objects can have a lot of text in them.
 *
 * The sizes here don't take NULL termination into account, so the value here
 * and the maxiumum length of the key/value is identical. 
 *
 * These only have an effect when we are parsing URL encoded data.*/
#define MAX_KEY_LEN           256
#define MAX_VALUE_LEN (64 * 1024)

/* Define the maximum multipart data a single form submission
 * can contain. This includes the data and the mime headers
 * and such. This roughly limits the maximum size of file that 
 * can be uploaded, while leaving space for the headers and other
 * form components.
 *
 * This implicitly limits how big keys and values in the submitted data
 * can be. As long as the encoded data fits into this buffer, we will
 * have no problems with the data.
 *
 * NOTE: Actual memory required is roughly double what is set here (at worst).
 * This is because we store a copy of the data before we give it to
 * lua, which will make it's own copy and allocate accordingly.
 *
 * Here we're setting 1.5 meg, which seems like a good compromise. */
#define MAX_MULTIPART_DATA ((1024 * 1024) + (1024 * 512))

/* Definitions of the content type encodings that we know about.
 *
 * url encoded data is either attached to the end of the URL submitted
 * (GET forms), or written to our stdin for us by the server (POST forms). 
 *
 * multipart data is a mime type encoding. This is only ever sent via
 * a POST form. This form of encoding allows uploading of files. */
#define URL_ENCODED_DATA "application/x-www-form-urlencoded"
#define MULTIPART_DATA   "multipart/form-data"

/*****************************************************************************/

/* Copies a string from one place to another in a special way. string is the 
 * string we want to copy out of. buffer is where we want to copy to. maxLen is
 * the maximum length (not counting null byte) that we want to copy.
 * stopChar is the character we want to assume is the end of the string.
 * 
 * The string is copied until maxLen bytes have been copied, the end of the
 * string has been reached, or stopChar is hit. The buffer is always
 * null terminated, but stopChar will not be copied.
 *
 * During copying, sequences like %XX, where XX is a hex value, are
 * replaced with their ascii value. Decoding of + as a space is also done.
 * No other special URL decoding is done.
 *
 * The return value is a pointer inside string where we stopped. This would
 * be either pointing at stopChar or at the terminating null byte on the
 * string. */
char *copyStr (char *string , char *buffer , int maxLen , char stopChar);

/* Open up the CGI query library. */
LUALIB_API void lua_querylibopen (lua_State *L);

/*****************************************************************************/

#endif


